#!/bin/sh
#
#	$Id: maj.sh,v 1.28 2011/06/13 22:17:13 mattieu Exp $
#
# Copyright (c) 2007-2011 Mattieu Baptiste <mattieu@brimbelle.org>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
# 

umask 0022

unset ARCH BRANCH JOBOPT KERNCONF NCPU VERSION WITHX DESTDIR RELEASEDIR

ARCH=`machine`
KERNCONF="GENERIC"
NCPU=`sysctl -n hw.ncpu`

if [ -f /etc/mk.conf ]; then
	if [ "`id -u`" -ne "0" ]; then
		SUDO=`grep -s '^SUDO' /etc/mk.conf | sed -e "s/^SUDO=//"`
	fi
	BSDSRCDIR=`grep -s '^BSDSRCDIR' /etc/mk.conf | sed -e "s/^BSDSRCDIR=//"`
	XSRCDIR=`grep -s '^XSRCDIR' /etc/mk.conf | sed -e "s/^XSRCDIR=//"`
	BSDOBJDIR=`grep -s '^BSDOBJDIR' /etc/mk.conf | sed -e "s/^BSDOBJDIR=//"`
	XOBJDIR=`grep -s '^XOBJDIR' /etc/mk.conf | sed -e "s/^XOBJDIR=//"`
	PORTSDIR=`grep -s '^PORTSDIR' /etc/mk.conf | sed -e "s/^PORTSDIR=//"`
fi

BSDSRCDIR="${BSDSRCDIR:=/usr/src}"
XSRCDIR="${XSRCDIR:=/usr/xenocara}"
BSDOBJDIR="${BSDOBJDIR:=/usr/obj}"
XOBJDIR="${XOBJDIR:=/usr/xobj}"
PORTSDIR="${PORTSDIR:=/usr/ports}"

BRANCH=`sysctl kern.version | sed -e "s/.*\`uname -r\`//" -e "s/ .*$//"`
if [ "${BRANCH}" = "-current" -o "${BRANCH}" = "-beta" ]; then
	VERSION=""
elif [ "${BRANCH}" = "-stable" -o "${BRANCH}" = "" ]; then
	VERSION="-rOPENBSD_"`uname -r | sed -e "s/\./_/"`
fi

if [ "${NCPU}" -gt "1" ]; then
	KERNCONF="GENERIC.MP"
	JOBOPT="-j ${NCPU}"
fi

if [ -x /usr/X11R6/bin/startx ]; then
	WITHX="YES"
fi

usage() {
	echo "usage: ${0##*/} [-itVX] [-C configfile] [-c component] [-D destdir] [-d cvsroot]" >&2
	echo "\t[-g tree] [-j njobs] [-p tree] [-r component] [-R releasedir] [-u tree]" >&2
	echo "\t[-k component]" >&2
	exit 1
}

trap 'echo " *** Error: ${0##*/} abnormally terminates"; exit 1' 1 2 3 13 15

perm_x() {
	echo "changing ${1} permissions..."
	case "${1}" in
	src)
		chgrp -R wsrc ${BSDSRCDIR}
		find ${BSDSRCDIR} -exec chmod g+w {} \;
		;;
	xenocara)
		chgrp -R wsrc ${XSRCDIR}
		find ${XSRCDIR} -exec chmod g+w {} \;
		;;
	ports)
		chgrp -R wsrc ${PORTSDIR}
		find ${PORTSDIR} -type d -exec chmod g+w {} \;
		;;
	esac
}

get_x() {
	echo "getting ${1}..."
	cd /usr && cvs -qd ${CVSROOT} get ${VERSION} -P ${1}
}

up_x() {
	echo "updating ${1}..."
	case "${1}" in
	src)
		cd ${BSDSRCDIR} && cvs -d ${CVSROOT} -q up ${VERSION} -Pd
		;;
	xenocara)
		cd ${XSRCDIR} && cvs -d ${CVSROOT} -q up ${VERSION} -Pd
		;;
	ports)
		cd ${PORTSDIR} && cvs -d ${CVSROOT} -q up ${VERSION} -Pd
		;;
	esac
}

rm_x() {
	echo "cleaning ${1}..."
	case "${1}" in
	kobj)
		cd ${BSDSRCDIR}/sys/arch/${ARCH}/compile && rm -rf ${KERNCONF}
		;;
	bsdobj)
		cd ${BSDOBJDIR} && mkdir -p .old && ${SUDO} mv * .old && ${SUDO} rm -rf .old
		;;
	xobj)
		cd ${XOBJDIR} && mkdir -p .old && ${SUDO} mv * .old && ${SUDO} rm -rf .old
		;;
	esac
}

comp_x() {
	case ${1} in
	kernel)
		cd ${BSDSRCDIR}/sys/arch/${ARCH}/conf
		/usr/sbin/config ${KERNCONF}
		cd ${BSDSRCDIR}/sys/arch/${ARCH}/compile/${KERNCONF}
		make ${JOBOPT} clean && make ${JOBOPT}
		;;
	uland)
		rm_x bsdobj
		cd ${BSDSRCDIR} && nice make ${JOBOPT} obj
		cd ${BSDSRCDIR}/etc && env DESTDIR=/ ${SUDO} make ${JOBOPT} distrib-dirs
		cd ${BSDSRCDIR} && nice make ${JOBOPT} SUDO=${SUDO} build
		;;
	xenocara)
		rm_x xobj
		cd ${XSRCDIR}
		${SUDO} make ${JOBOPT} bootstrap
		${SUDO} make ${JOBOPT} obj
		${SUDO} make ${JOBOPT} build
		;;
	esac
}

install_kernel() {
	cd ${BSDSRCDIR}/sys/arch/${ARCH}/compile/${KERNCONF}
	${SUDO} make install
}

make_release() {
	${SUDO} sh -c "test -d ${DESTDIR} && mv ${DESTDIR} ${DESTDIR}- && rm -rf ${DESTDIR}- &"
	${SUDO} sh -c "mkdir -p ${DESTDIR} ${RELEASEDIR}"
	case ${1} in
	src)
		${SUDO} sh -c "cd ${BSDSRCDIR}/etc && make release"
		${SUDO} sh -c "cd ${BSDSRCDIR}/distrib/sets && sh checkflist"
		;;
	xenocara)
		${SUDO} sh -c "cd ${XSRCDIR} && make release"
		;;
	esac
}

while getopts 'C:c:D:d:g:j:k:p:r:R:u:itVX' arg; do
	case ${arg} in
	C)
		if [ -f "${BSDSRCDIR}/sys/arch/${ARCH}/conf/${OPTARG}" ]; then
			KERNCONF=${OPTARG}
		else
			echo " *** Error: the kernel configuration file doesn't exist"
			exit 1
		fi
		;;
	c)
		if [ "${OPTARG}" == "kernel" -o "${OPTARG}" == "uland" -o "${OPTARG}" == "xenocara" -o "${OPTARG}" == "all" ]; then
			compileopt=${OPTARG}
		else
			echo " *** Error: the component to compile is not valid"
			exit 1
		fi
		;;
	D)
		DESTDIR=${OPTARG}
		;;
	d)
		if echo -n "${OPTARG}" | grep '^anoncvs\@.*:/cvs$' > /dev/null 2>&1 ; then
			CVSROOT=${OPTARG}
		else
			echo " *** Error: your anonymous CVS is not valid"
			exit 1
		fi
		;;
	g)
		if [ "${OPTARG}" == "src" -o "${OPTARG}" == "xenocara" -o "${OPTARG}" == "ports" -o "${OPTARG}" == "all" ]; then
			getopt=${OPTARG}
		else
			echo " *** Error: the tree to get is not valid"
			exit 1
		fi
		;;
	i)
		installopt=1
		;;
	j)
		if [ ${OPTARG} -gt "0" ]; then
			JOBOPT="-j ${OPTARG}"
		elif [ ${OPTARG} -eq "0" ]; then
			JOBOPT=""
		else
			echo " *** Error: please provide 0 or more jobs"
			exit 1
		fi
		;;
	k)
		if [ "${OPTARG}" == "kobj" -o "${OPTARG}" == "bsdobj" -o "${OPTARG}" == "xobj" -o "${OPTARG}" == "all" ]; then
			cleanopt=${OPTARG}
		else
			echo " *** Error: the component to clean is not valid"
			exit 1
		fi
		;;
	p)
		if [ "${OPTARG}" == "src" -o "${OPTARG}" == "xenocara" -o "${OPTARG}" == "ports" -o "${OPTARG}" == "all" ]; then
			permopt=${OPTARG}
		else
			echo " *** Error: the tree for which to change permissions is not valid"
			exit 1
		fi
		;;
	R)
		RELEASEDIR=${OPTARG}
		;;
	r)
		if [ "${OPTARG}" == "src" -o "${OPTARG}" == "xenocara" -o "${OPTARG}" == "all" ]; then
			releaseopt=${OPTARG}
		else
			echo " *** Error: the component to release is not valid"
			exit 1
		fi
		;;
	t)
		timeopt=1
		;;
	u)
		if [ "${OPTARG}" == "src" -o "${OPTARG}" == "xenocara" -o "${OPTARG}" == "ports" -o "${OPTARG}" == "all" ]; then
			updateopt=${OPTARG}
		else
			echo " *** Error: the tree to update is not valid"
			exit 1
		fi
		;;
	V)
		VERSION=""
		;;
	X)
		WITHX="YES"
		;;
	\?)
		usage
		;;
	esac
done

if [ "${OPTIND}" -lt "2" ]; then
	usage
fi

if [ "${permopt}" ]; then
	case ${permopt} in
	src | ports | xenocara)
		perm_x ${permopt}
		;;
	all)
		perm_x src
		if [ "${WITHX}" ]; then
			perm_x xenocara
		fi
		perm_x ports
		;;
	esac
fi

if [ "${getopt}" ]; then
	case ${getopt} in
	src | ports | xenocara)
		get_x ${getopt}
		;;
	all)
		get_x src
		if [ "${WITHX}" ]; then
			get_x xenocara
		fi
		get_x ports
		;;
	esac
fi

if [ "${updateopt}" ]; then
	case ${updateopt} in
	src | ports | xenocara)
		up_x ${updateopt}
		;;
	all)
		up_x src
		if [ "${WITHX}" ]; then
			up_x xenocara
		fi
		up_x ports
		;;
	esac
fi

if [ "${cleanopt}" ]; then
	case ${cleanopt} in
	kobj | bsdobj | xobj)
		rm_x ${cleanopt}
		;;
	all)
		rm_x kobj
		rm_x bsdobj
		if [ "${WITHX}" ]; then
			rm_x xobj
		fi
		;;
	esac
fi

if [ "${compileopt}" ]; then
	time=`date +"%Y-%m-%d %T"`
	case ${compileopt} in
	kernel | uland | xenocara)
		comp_x ${compileopt}
		;;
	all)
		comp_x kernel
		install_kernel
		comp_x uland
		if [ "${WITHX}" ]; then
			comp_x xenocara
		fi
		;;
	esac
	if [ "${timeopt}" ]; then
		echo "Started:"'\t'${time}
		echo -n "Ended:"'\t''\t'
		date +"%Y-%m-%d %T"
	fi
fi

if [ "${installopt}" ]; then
	install_kernel
fi

if [ "${releaseopt}" ]; then
	if [ -z "${DESTDIR}" -o -z "${RELEASEDIR}" ]; then
		echo " *** Error: check DESTDIR and RELEASEDIR"
		exit 1
	fi
	export RELEASEDIR DESTDIR
	time=`date +"%Y-%m-%d %T"`
	case ${releaseopt} in
	src | xenocara )
		make_release ${releaseopt}
		;;
	all)
		make_release src
		if [ "${WITHX}" ]; then
			make_release xenocara
		fi
		;;
	esac
	if [ "${timeopt}" ]; then
		echo "Started:"'\t'${time}
		echo -n "Ended:"'\t''\t'
		date +"%Y-%m-%d %T"
	fi
	unset RELEASEDIR DESTDIR
fi
